@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -25,8 +25,8 @@ struct DisplayServerPrivate
     /* Unique name for this display server */
     gchar *name;
 
-    /* TRUE if sessions should be automatically started on this display server */
-    gboolean start_local_sessions;
+    /* TRUE when started */
+    gboolean is_ready;
 
     /* TRUE when being stopped */
     gboolean stopping;
@@ -35,7 +35,11 @@ struct DisplayServerPrivate
     gboolean stopped;
 };
 
-G_DEFINE_TYPE (DisplayServer, display_server, G_TYPE_OBJECT);
+static void display_server_logger_iface_init (LoggerInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (DisplayServer, display_server, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (
+                             LOGGER_TYPE, display_server_logger_iface_init));
 
 void
 display_server_set_name (DisplayServer *server, const gchar *name)
@@ -52,18 +56,47 @@ display_server_get_name (DisplayServer *server)
     return server->priv->name;
 }
 
-void
-display_server_set_start_local_sessions (DisplayServer *server, gboolean start_local_sessions)
+const gchar *
+display_server_get_session_type (DisplayServer *server)
 {
-    g_return_if_fail (server != NULL);
-    server->priv->start_local_sessions = start_local_sessions;
+    return DISPLAY_SERVER_GET_CLASS (server)->get_session_type (server);
+}
+
+DisplayServer *
+display_server_get_parent (DisplayServer *server)
+{
+    return DISPLAY_SERVER_GET_CLASS (server)->get_parent (server);
+}
+
+static DisplayServer *
+display_server_real_get_parent (DisplayServer *server)
+{
+    return NULL;
 }
 
 gboolean
-display_server_get_start_local_sessions (DisplayServer *server)
+display_server_get_can_share (DisplayServer *server)
 {
-    g_return_val_if_fail (server != NULL, FALSE);
-    return server->priv->start_local_sessions;
+    return DISPLAY_SERVER_GET_CLASS (server)->get_can_share (server);
+}
+
+static gboolean
+display_server_real_get_can_share (DisplayServer *server)
+{
+    return FALSE;
+}
+
+gint
+display_server_get_vt (DisplayServer *server)
+{
+    g_return_val_if_fail (server != NULL, -1);
+    return DISPLAY_SERVER_GET_CLASS (server)->get_vt (server);
+}
+
+static gint
+display_server_real_get_vt (DisplayServer *server)
+{
+    return -1;
 }
 
 gboolean
@@ -73,13 +106,43 @@ display_server_start (DisplayServer *server)
     return DISPLAY_SERVER_GET_CLASS (server)->start (server);
 }
 
+gboolean
+display_server_get_is_ready (DisplayServer *server)
+{
+    g_return_val_if_fail (server != NULL, FALSE);
+    return server->priv->is_ready;
+}
+
 static gboolean
 display_server_real_start (DisplayServer *server)
 {
+    server->priv->is_ready = TRUE;
     g_signal_emit (server, signals[READY], 0);
     return TRUE;
 }
 
+void
+display_server_connect_session (DisplayServer *server, Session *session)
+{
+    return DISPLAY_SERVER_GET_CLASS (server)->connect_session (server, session);
+}
+
+static void
+display_server_real_connect_session (DisplayServer *server, Session *session)
+{
+}
+
+void
+display_server_disconnect_session (DisplayServer *server, Session *session)
+{
+    return DISPLAY_SERVER_GET_CLASS (server)->disconnect_session (server, session);
+}
+
+static void
+display_server_real_disconnect_session (DisplayServer *server, Session *session)
+{
+}
+
 void
 display_server_stop (DisplayServer *server)
 {
@@ -93,16 +156,15 @@ display_server_stop (DisplayServer *server)
 }
 
 gboolean
-display_server_get_is_stopped (DisplayServer *server)
+display_server_get_is_stopping (DisplayServer *server)
 {
-    g_return_val_if_fail (server != NULL, TRUE);
-    return server->priv->stopped;
+    g_return_val_if_fail (server != NULL, FALSE);
+    return server->priv->stopping;
 }
 
 static void
 display_server_real_stop (DisplayServer *server)
 {
-    server->priv->stopped = TRUE;
     g_signal_emit (server, signals[STOPPED], 0);
 }
 
@@ -110,31 +172,65 @@ static void
 display_server_init (DisplayServer *server)
 {
     server->priv = G_TYPE_INSTANCE_GET_PRIVATE (server, DISPLAY_SERVER_TYPE, DisplayServerPrivate);
-    server->priv->start_local_sessions = TRUE;
+}
+
+static void
+display_server_finalize (GObject *object)
+{
+    DisplayServer *self = DISPLAY_SERVER (object);
+
+    g_free (self->priv->name);
+
+    G_OBJECT_CLASS (display_server_parent_class)->finalize (object);
 }
 
 static void
 display_server_class_init (DisplayServerClass *klass)
 {
+    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+    klass->get_parent = display_server_real_get_parent;  
+    klass->get_can_share = display_server_real_get_can_share;
+    klass->get_vt = display_server_real_get_vt;
     klass->start = display_server_real_start;
+    klass->connect_session = display_server_real_connect_session;
+    klass->disconnect_session = display_server_real_disconnect_session;
     klass->stop = display_server_real_stop;
+    object_class->finalize = display_server_finalize;
 
     g_type_class_add_private (klass, sizeof (DisplayServerPrivate));
 
     signals[READY] =
-        g_signal_new ("ready",
+        g_signal_new (DISPLAY_SERVER_SIGNAL_READY,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (DisplayServerClass, ready),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
+                      NULL,
                       G_TYPE_NONE, 0);
     signals[STOPPED] =
-        g_signal_new ("stopped",
+        g_signal_new (DISPLAY_SERVER_SIGNAL_STOPPED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (DisplayServerClass, stopped),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
+                      NULL,
                       G_TYPE_NONE, 0);
 }
+
+static gint
+display_server_real_logprefix (Logger *self, gchar *buf, gulong buflen)
+{
+    DisplayServer *server = DISPLAY_SERVER (self);
+    const gchar *name = display_server_get_name (server);
+    if (name)
+        return g_snprintf (buf, buflen, "DisplayServer %s: ", name);
+    else
+        return g_snprintf (buf, buflen, "DisplayServer: ");
+}
+
+static void
+display_server_logger_iface_init (LoggerInterface *iface)
+{
+    iface->logprefix = &display_server_real_logprefix;
+}
