@@ -40,21 +40,19 @@ seat_xremote_create_display_server (Seat *seat, Session *session)
     }
 
     hostname = seat_get_string_property (seat, "xserver-hostname");
-    if (!hostname)
-        hostname = "localhost";
     number = seat_get_integer_property (seat, "xserver-display-number");
 
-    l_debug (seat, "Starting remote X display %s:%d", hostname, number);
+    l_debug (seat, "Starting remote X display %s:%d", hostname ? hostname : "", number);
 
     x_server = x_server_remote_new (hostname, number, NULL);
 
     return DISPLAY_SERVER (x_server);
 }
 
-static Greeter *
+static GreeterSession *
 seat_xremote_create_greeter_session (Seat *seat)
 {
-    Greeter *greeter_session;
+    GreeterSession *greeter_session;
 
     greeter_session = SEAT_CLASS (seat_xremote_parent_class)->create_greeter_session (seat);
     session_set_env (SESSION (greeter_session), "XDG_SEAT", seat_get_name (seat));
