@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -12,7 +12,6 @@
 #include <config.h>
 #include <string.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <glib/gstdio.h>
@@ -31,18 +30,18 @@ struct XServerLocalPrivate
     /* Command to run the X server */
     gchar *command;
 
+    /* Display number to use */
+    guint display_number;
+
     /* Config file to use */
     gchar *config_file;
 
     /* Server layout to use */
     gchar *layout;
-    
+
     /* Value for -seat argument */
     gchar *xdg_seat;
 
-    /* TRUE if should share VT with other X server */
-    gboolean sharevts;
-
     /* TRUE if TCP/IP connections are allowed */
     gboolean allow_tcp;
 
@@ -58,12 +57,6 @@ struct XServerLocalPrivate
     /* XDMCP key to use */
     gchar *xdmcp_key;
 
-    /* ID to report to Mir */
-    gchar *mir_id;
-
-    /* Filename of socket Mir is listening on */
-    gchar *mir_socket;
-
     /* TRUE when received ready signal */
     gboolean got_signal;
 
@@ -77,8 +70,65 @@ struct XServerLocalPrivate
 
 G_DEFINE_TYPE (XServerLocal, x_server_local, X_SERVER_TYPE);
 
+static gchar *version = NULL;
+static guint version_major = 0, version_minor = 0;
 static GList *display_numbers = NULL;
 
+#define XORG_VERSION_PREFIX "X.Org X Server "
+
+static gchar *
+find_version (const gchar *line)
+{
+    if (!g_str_has_prefix (line, XORG_VERSION_PREFIX))
+        return NULL;
+
+    return g_strdup (line + strlen (XORG_VERSION_PREFIX));
+}
+
+const gchar *
+x_server_local_get_version (void)
+{
+    gchar *stderr_text;
+    gint exit_status;
+    gchar **tokens;
+    guint n_tokens;
+
+    if (version)
+        return version;
+
+    if (!g_spawn_command_line_sync ("X -version", NULL, &stderr_text, &exit_status, NULL))
+        return NULL;
+    if (exit_status == EXIT_SUCCESS)
+    {
+        gchar **lines;
+        int i;
+
+        lines = g_strsplit (stderr_text, "\n", -1);
+        for (i = 0; lines[i] && !version; i++)
+            version = find_version (lines[i]);
+        g_strfreev (lines);
+    }
+    g_free (stderr_text);
+
+    tokens = g_strsplit (version, ".", 3);
+    n_tokens = g_strv_length (tokens);
+    version_major = n_tokens > 0 ? atoi (tokens[0]) : 0;
+    version_minor = n_tokens > 1 ? atoi (tokens[1]) : 0;
+    g_strfreev (tokens);
+
+    return version;
+}
+
+gint
+x_server_local_version_compare (guint major, guint minor)
+{
+    x_server_local_get_version ();
+    if (major == version_major)
+        return version_minor - minor;
+    else
+        return version_major - major;
+}
+
 static gboolean
 display_number_in_use (guint display_number)
 {
@@ -113,7 +163,7 @@ display_number_in_use (guint display_number)
     }
 
     g_free (path);
-  
+
     return in_use;
 }
 
@@ -150,20 +200,10 @@ XServerLocal *
 x_server_local_new (void)
 {
     XServerLocal *self;
-    gchar hostname[1024], *number, *name;
-    XAuthority *cookie;
+    gchar *name;
 
     self = g_object_new (X_SERVER_LOCAL_TYPE, NULL);
 
-    x_server_set_display_number (X_SERVER (self), x_server_local_get_unused_display_number ());
-
-    gethostname (hostname, 1024);
-    number = g_strdup_printf ("%d", x_server_get_display_number (X_SERVER (self)));
-    cookie = x_authority_new_cookie (XAUTH_FAMILY_LOCAL, (guint8*) hostname, strlen (hostname), number);
-    x_server_set_authority (X_SERVER (self), cookie);
-    g_free (number);
-    g_object_unref (cookie);
-
     name = g_strdup_printf ("x-%d", x_server_get_display_number (X_SERVER (self)));
     display_server_set_name (DISPLAY_SERVER (self), name);
     g_free (name);
@@ -218,13 +258,6 @@ x_server_local_set_xdg_seat (XServerLocal *server, const gchar *xdg_seat)
     server->priv->xdg_seat = g_strdup (xdg_seat);
 }
 
-void
-x_server_local_set_sharevts (XServerLocal *server, gboolean sharevts)
-{
-    g_return_if_fail (server != NULL);
-    server->priv->sharevts = sharevts;
-}
-
 void
 x_server_local_set_allow_tcp (XServerLocal *server, gboolean allow_tcp)
 {
@@ -278,26 +311,10 @@ x_server_local_set_background (XServerLocal *server, const gchar *background)
     server->priv->background = g_strdup (background);
 }
 
-void
-x_server_local_set_mir_id (XServerLocal *server, const gchar *id)
+static guint
+x_server_local_get_display_number (XServer *server)
 {
-    g_return_if_fail (server != NULL);
-    g_free (server->priv->mir_id);
-    server->priv->mir_id = g_strdup (id);
-}
-
-const gchar *x_server_local_get_mir_id (XServerLocal *server)
-{
-    g_return_val_if_fail (server != NULL, NULL);
-    return server->priv->mir_id; 
-}
-
-void
-x_server_local_set_mir_socket (XServerLocal *server, const gchar *socket)
-{
-    g_return_if_fail (server != NULL);
-    g_free (server->priv->mir_socket);
-    server->priv->mir_socket = g_strdup (socket);
+    return X_SERVER_LOCAL (server)->priv->display_number;
 }
 
 static gint
@@ -338,7 +355,7 @@ get_absolute_command (const gchar *command)
 }
 
 static void
-run_cb (Process *process, gpointer user_data)
+x_server_local_run (Process *process, gpointer user_data)
 {
     int fd;
 
@@ -351,13 +368,25 @@ run_cb (Process *process, gpointer user_data)
     signal (SIGUSR1, SIG_IGN);
 }
 
+static ProcessRunFunc
+x_server_local_get_run_function (XServerLocal *server)
+{
+    return x_server_local_run;
+}
+
+static gboolean
+x_server_local_get_log_stdout (XServerLocal *server)
+{
+    return TRUE;
+}
+
 static void
 got_signal_cb (Process *process, int signum, XServerLocal *server)
 {
     if (signum == SIGUSR1 && !server->priv->got_signal)
     {
         server->priv->got_signal = TRUE;
-        l_debug (server, "Got signal from X server :%d", x_server_get_display_number (X_SERVER (server)));
+        l_debug (server, "Got signal from X server :%d", server->priv->display_number);
 
         // FIXME: Check return value
         DISPLAY_SERVER_CLASS (x_server_local_parent_class)->start (DISPLAY_SERVER (server));
@@ -375,8 +404,8 @@ stopped_cb (Process *process, XServerLocal *server)
         vt_unref (server->priv->vt);
         server->priv->have_vt_ref = FALSE;
     }
-    x_server_local_release_display_number (x_server_get_display_number (X_SERVER (server)));
-  
+    x_server_local_release_display_number (server->priv->display_number);
+
     if (x_server_get_authority (X_SERVER (server)) && server->priv->authority_file)
     {
         l_debug (server, "Removing X server authority %s", server->priv->authority_file);
@@ -404,7 +433,7 @@ write_authority_file (XServerLocal *server)
     if (!server->priv->authority_file)
     {
         gchar *run_dir, *dir;
-      
+
         run_dir = config_get_string (config_get_instance (), "LightDM", "run-directory");
         dir = g_build_filename (run_dir, "root", NULL);
         g_free (run_dir);
@@ -427,7 +456,8 @@ static gboolean
 x_server_local_start (DisplayServer *display_server)
 {
     XServerLocal *server = X_SERVER_LOCAL (display_server);
-    gboolean result;
+    ProcessRunFunc run_cb;
+    gboolean result, backup_logs;
     gchar *filename, *dir, *log_file, *absolute_command;
     GString *command;
 
@@ -437,16 +467,18 @@ x_server_local_start (DisplayServer *display_server)
 
     g_return_val_if_fail (server->priv->command != NULL, FALSE);
 
+    run_cb = X_SERVER_LOCAL_GET_CLASS (server)->get_run_function (server);
     server->priv->x_server_process = process_new (run_cb, server);
     process_set_clear_environment (server->priv->x_server_process, TRUE);
-    g_signal_connect (server->priv->x_server_process, "got-signal", G_CALLBACK (got_signal_cb), server);
-    g_signal_connect (server->priv->x_server_process, "stopped", G_CALLBACK (stopped_cb), server);
+    g_signal_connect (server->priv->x_server_process, PROCESS_SIGNAL_GOT_SIGNAL, G_CALLBACK (got_signal_cb), server);
+    g_signal_connect (server->priv->x_server_process, PROCESS_SIGNAL_STOPPED, G_CALLBACK (stopped_cb), server);
 
     /* Setup logging */
     filename = g_strdup_printf ("%s.log", display_server_get_name (display_server));
     dir = config_get_string (config_get_instance (), "LightDM", "log-directory");
     log_file = g_build_filename (dir, filename, NULL);
-    process_set_log_file (server->priv->x_server_process, log_file, TRUE);
+    backup_logs = config_get_boolean (config_get_instance (), "LightDM", "backup-logs");
+    process_set_log_file (server->priv->x_server_process, log_file, X_SERVER_LOCAL_GET_CLASS (server)->get_log_stdout (server), backup_logs ? LOG_MODE_BACKUP_AND_TRUNCATE : LOG_MODE_APPEND);
     l_debug (display_server, "Logging to %s", log_file);
     g_free (log_file);
     g_free (filename);
@@ -462,31 +494,21 @@ x_server_local_start (DisplayServer *display_server)
     command = g_string_new (absolute_command);
     g_free (absolute_command);
 
-    g_string_append_printf (command, " :%d", x_server_get_display_number (X_SERVER (server)));
+    g_string_append_printf (command, " :%d", server->priv->display_number);
 
     if (server->priv->config_file)
         g_string_append_printf (command, " -config %s", server->priv->config_file);
 
     if (server->priv->layout)
         g_string_append_printf (command, " -layout %s", server->priv->layout);
-        
+
     if (server->priv->xdg_seat)
         g_string_append_printf (command, " -seat %s", server->priv->xdg_seat);
 
-    if (server->priv->sharevts)
-        g_string_append (command, " -sharevts");
-
     write_authority_file (server);
     if (server->priv->authority_file)
         g_string_append_printf (command, " -auth %s", server->priv->authority_file);
 
-    /* Setup for running inside Mir */
-    if (server->priv->mir_id)
-        g_string_append_printf (command, " -mir %s", server->priv->mir_id);
-
-    if (server->priv->mir_socket)
-        g_string_append_printf (command, " -mirSocket %s", server->priv->mir_socket);
-
     /* Connect to a remote server using XDMCP */
     if (server->priv->xdmcp_server != NULL)
     {
@@ -496,7 +518,12 @@ x_server_local_start (DisplayServer *display_server)
         if (server->priv->xdmcp_key)
             g_string_append_printf (command, " -cookie %s", server->priv->xdmcp_key);
     }
-    else if (!server->priv->allow_tcp)
+    else if (server->priv->allow_tcp)
+    {
+        if (x_server_local_version_compare (1, 17) >= 0)
+            g_string_append (command, " -listen tcp");
+    }
+    else
         g_string_append (command, " -nolisten tcp");
 
     if (server->priv->vt >= 0)
@@ -505,6 +532,10 @@ x_server_local_start (DisplayServer *display_server)
     if (server->priv->background)
         g_string_append_printf (command, " -background %s", server->priv->background);
 
+    /* Allow sub-classes to add arguments */
+    if (X_SERVER_LOCAL_GET_CLASS (server)->add_args)
+        X_SERVER_LOCAL_GET_CLASS (server)->add_args (server, command);
+
     process_set_command (server->priv->x_server_process, command->str);
     g_string_free (command, TRUE);
 
@@ -536,14 +567,14 @@ x_server_local_start (DisplayServer *display_server)
     result = process_start (server->priv->x_server_process, FALSE);
 
     if (result)
-        l_debug (display_server, "Waiting for ready signal from X server :%d", x_server_get_display_number (X_SERVER (server)));
+        l_debug (display_server, "Waiting for ready signal from X server :%d", server->priv->display_number);
 
     if (!result)
         stopped_cb (server->priv->x_server_process, X_SERVER_LOCAL (server));
 
     return result;
 }
- 
+
 static void
 x_server_local_stop (DisplayServer *server)
 {
@@ -556,16 +587,15 @@ x_server_local_init (XServerLocal *server)
     server->priv = G_TYPE_INSTANCE_GET_PRIVATE (server, X_SERVER_LOCAL_TYPE, XServerLocalPrivate);
     server->priv->vt = -1;
     server->priv->command = g_strdup ("X");
+    server->priv->display_number = x_server_local_get_unused_display_number ();
 }
 
 static void
 x_server_local_finalize (GObject *object)
 {
-    XServerLocal *self;
+    XServerLocal *self = X_SERVER_LOCAL (object);
 
-    self = X_SERVER_LOCAL (object);  
-
-    if (self->priv->x_server_process) 
+    if (self->priv->x_server_process)
     {
         g_signal_handlers_disconnect_matched (self->priv->x_server_process, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
         g_object_unref (self->priv->x_server_process);
@@ -576,8 +606,6 @@ x_server_local_finalize (GObject *object)
     g_free (self->priv->xdg_seat);
     g_free (self->priv->xdmcp_server);
     g_free (self->priv->xdmcp_key);
-    g_free (self->priv->mir_id);
-    g_free (self->priv->mir_socket);
     g_free (self->priv->authority_file);
     if (self->priv->have_vt_ref)
         vt_unref (self->priv->vt);
@@ -590,10 +618,14 @@ static void
 x_server_local_class_init (XServerLocalClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    XServerClass *x_server_class = X_SERVER_CLASS (klass);
     DisplayServerClass *display_server_class = DISPLAY_SERVER_CLASS (klass);
 
+    klass->get_run_function = x_server_local_get_run_function;
+    klass->get_log_stdout = x_server_local_get_log_stdout;
+    x_server_class->get_display_number = x_server_local_get_display_number;
     display_server_class->get_vt = x_server_local_get_vt;
-    display_server_class->start = x_server_local_start;
+    display_server_class->start = klass->start = x_server_local_start;
     display_server_class->stop = x_server_local_stop;
     object_class->finalize = x_server_local_finalize;
 
